<?php

declare(strict_types=1);

namespace Drupal\menu_parser_php;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Menu\MenuLinkTreeElement;
use GuzzleHttp\Client;

/**
 * Parses a remote menu JSON linkset.
 */
class RemoteMenuParser {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected Client $httpClient;

  /**
   * The module config.
   *
   * @var Config
   */
  protected Config $config;

  /**
   * A custom cache bin.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected CacheBackendInterface $cacheBin;

  /**
   * Time service.
   *
   * @var TimeInterface
   */
  protected TimeInterface $time;

  /**
   * RemoteMenuParser constructor.
   */
  public function __construct(
    Client $client,
    ConfigFactoryInterface $config_factory,
    CacheBackendInterface $cache,
    TimeInterface $time
  ) {
    $this->httpClient = $client;
    $this->config = $config_factory->get('menu_parser_php.settings');
    $this->cacheBin = $cache;
    $this->time = $time;
  }

  /**
   * Parses a remote menu.
   *
   * @param string $url
   *   The URL at which the JSON linkset is available.
   * @return MenuLinkTreeElement[]|null
   *   A tree of MenuLinkTreeElement objects. NULL on failure.
   */
  public function readRemoteMenu(string $url): ?array {
    $cached = $this->cacheBin->get($url);
    if ($cached) {
      return $cached->data;
    }

    try {
      $json = $this->httpClient->get($url)->getBody()->getContents();
      $json_parsed = json_decode($json, FALSE, 512, JSON_THROW_ON_ERROR);
    }
    catch (\Exception $e) {
      return NULL;
    }
    $data = [];
    // Convert a flat JSON into a tree array.
    foreach ($json_parsed->linkset[0]->item as $item) {
      $data = static::insertLink($item->{'drupal-menu-hierarchy'}[0], $item->href, $item->title, $data);
    }
    // Convert the tree array into a tree of RemoteMenuLink objects.
    static::convertToObjects($data);

    $expire_time_config = $this->config->get('cache_expire') ?? 24 * 60 * 60;
    $expire_time_config = (int) $expire_time_config;
    if ($expire_time_config === CacheBackendInterface::CACHE_PERMANENT) {
      $expire = CacheBackendInterface::CACHE_PERMANENT;
    }
    else {
      $expire = $this->time->getCurrentTime() + $expire_time_config;
    }
    $this->cacheBin->set($url, $data, $expire);
    return $data;
  }

  /**
   * Converts the tree array into a tree of RemoteMenuLink objects.
   *
   * @param array $input
   *   The tree of links.
   */
  protected static function convertToObjects(array &$input): void {
    foreach ($input as &$item) {
      $weight = 0;
      if (array_key_exists('subtree', $item)) {
        static::convertToObjects($item['subtree']);
      }
      else {
        $item['subtree'] = [];
      }
      $link = RemoteMenuLink::fromLink($item['link'], $item['title'], $weight, $item['location']);
      $has_children = !empty($item['subtree']);
      $depth = mb_substr_count($item['location'], '.');
      $item = new MenuLinkTreeElement($link, $has_children, $depth, FALSE, $item['subtree']);
    }
  }

  /**
   * Creates a tree leaf. The leaf is an array.
   *
   * @param string $location
   *   The linkset item location.
   * @param string $link
   *   The menu link path.
   * @param string $title
   *   The menu link title.
   * @param array $data
   *   The tree into which the leaf will be inserted.
   *
   * @return array
   *   The adjusted tree.
   */
  protected static function insertLink(string $location, string $link, string $title, array $data): array {
    $pointer = &$data;
    $fragments = array_filter(explode('.', $location));
    // Traverse the tree.
    while (TRUE) {
      if (count($fragments) === 1) {
        // This is not the final child.
        $loc = reset($fragments);
        // Set the data of the child.
        $pointer[$loc]['link'] = $link;
        $pointer[$loc]['title'] = $title;
        $pointer[$loc]['location'] = $location;
        break;
      }
      // Shift out the topmost parent.
      $loc = array_shift($fragments);
      // Move the pointer into the child.
      $pointer = &$pointer[$loc]['subtree'];
    }
    return $data;
  }

}
